// Copyright (c) 2023 Tudor Stanciu

import { useContext, useCallback, useEffect, useMemo } from "react";
import { TuitioClient } from "@flare/tuitio-client";
import type { TuitioUserInfo, UserRole } from "@flare/tuitio-client";
import { TuitioContext, TuitioDispatchContext } from "../contexts";
import { userRoles } from "../constants/userRoles";

type UserFlags = {
  isSysAdmin: boolean;
  isDeveloper: boolean;
  isPowerUser: boolean;
  isRegularUser: boolean;
  isViewer: boolean;
  isGuest: boolean;
  isDemoGuest: boolean;
};

const defaultUserFlags: UserFlags = {
  isSysAdmin: false,
  isDeveloper: false,
  isPowerUser: false,
  isRegularUser: false,
  isViewer: false,
  isGuest: false,
  isDemoGuest: false
};

export type TuitioUserInfoHookResult = UserFlags & {
  userInfo?: TuitioUserInfo;
};

const getFlags = (roles: UserRole[]): UserFlags => {
  const isSysAdmin = roles.some(role => role.code === userRoles.SYS_ADMIN);
  const isDeveloper = roles.some(role => role.code === userRoles.DEVELOPER);
  const isPowerUser = roles.some(role => role.code === userRoles.POWER_USER);
  const isRegularUser = roles.some(role => role.code === userRoles.REGULAR_USER);
  const isViewer = roles.some(role => role.code === userRoles.VIEWER);
  const isGuest = roles.some(role => role.code === userRoles.GUEST);
  const isDemoGuest = roles.some(role => role.code === userRoles.DEMO);
  return { isSysAdmin, isDeveloper, isPowerUser, isRegularUser, isViewer, isGuest, isDemoGuest };
};

const useTuitioUserInfo = (): TuitioUserInfoHookResult => {
  const state = useContext(TuitioContext);
  const dispatchActions = useContext(TuitioDispatchContext);

  const getUserInfo = useCallback(async (): Promise<TuitioUserInfo> => {
    const client = new TuitioClient(state.configuration.tuitioUrl);
    const userInfo = await client.getUserInfo();
    dispatchActions.onUserInfoLoaded(userInfo);
    return userInfo;
  }, [dispatchActions, state.configuration.tuitioUrl]);

  useEffect(() => {
    if (state.userInfo) return;
    getUserInfo();
  }, [getUserInfo, state.userInfo]);

  const userInfo = state.userInfo;
  const flags = useMemo(() => (userInfo ? getFlags(userInfo.userRoles) : defaultUserFlags), [userInfo]);

  return { userInfo, ...flags };
};

export { useTuitioUserInfo };
